VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "clsWordFromRTB"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

' ------------------------------------------------------------------
'                   Klassenmodul clsWordFromRTB
' ------------------------------------------------------------------
'
' Zweck:  Erkennung eines Wortes in einer RichTextBox auf Basis
'         vorgegebener Koordinaten (z.B. Cursorposition).
'
' Anm.:   Selbige Methodik funktioniert auch mit Standard-Textboxen,
'         lediglich der Typ des Parameters ctlRTB mte dazu von
'         RichTextBox auf TextBox gendert werden.
'
' Stand:  11/98, Tips&Tricks der BasicPro 6/98
'
' ------------------------------------------------------------------
'                        ffentliche Elemente:
' ------------------------------------------------------------------
'
' Funktionen:
'
'  - WordFromPoint(ctlRTB As RichTextBox, _
'                  ByVal X As Single, _
'                  ByVal Y As Single) As String
'    Liefert das Wort zurck, das sich unter den Koordinaten
'    (X,Y) befindet, falls zutreffend (sonst vbNullString).
'
'    Parameter:  Bedeutung:
'    ctlRTB      RichTextBox-Control
'    X, Y        Koordinaten des Punkts (Skala: Twips)
'
' ------------------------------------------------------------------
'           Copyright  1998 Mathias Schiffer fr BasicPro
'             BasicPro Online: http://www.basicworld.com
' ------------------------------------------------------------------

' -----
' TYPEN
' -----
Private Type POINTAPI
    x As Long
    y As Long
End Type

' ---------
' KONSTANTE
' ---------
Private Const EM_CHARFROMPOS& = &HD7

' --------------
' API-FUNKTIONEN
' --------------
Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long

' --------
' VARIABLE
' --------
Private mvarCharsAllowed As String


' ------------------------------------------------------------------


Public Function WordFromPoint(ctlRTB As RichTextBox, ByVal x As Single, ByVal y As Single) As String
Dim PT As POINTAPI, pos As Integer, pos1 As Integer, pos2 As Integer
Dim strChar As String, strText As String, intLaenge As Integer
  
  ' Koordinaten der RTB sind immer Twips, das API jedoch arbeitet
  ' bei Koordinastenangaben grundstzlich mit Pixeln:
  PT.x = x \ Screen.TwipsPerPixelX
  PT.y = y \ Screen.TwipsPerPixelY
  
  ' Mittels SendMessage den Buchstaben bestimmen, der sich
  ' unter der angegebenen Position befindet
  pos = SendMessage(ctlRTB.hwnd, EM_CHARFROMPOS, 0&, PT)
  If pos <= 0 Then Exit Function
  
  strText = ctlRTB.Text
  
  ' Prfen, ob unter der Cursorposition ein Zeichen ist,
  ' das als Teil eines Wortes zulssig ist
  strChar = Mid$(ctlRTB.Text, pos, 1)
  If InStr(mvarCharsAllowed, strChar) = 0 Then Exit Function
  
  ' Beginn des Wortes suchen
  
  ' 1) Von Position aus Worbeginn suchen
  For pos1 = pos To 1 Step -1
    strChar = Mid$(ctlRTB.Text, pos1, 1)
    ' Ist dieser Buchstabe Teil eines Wortes?
    If InStr(mvarCharsAllowed, strChar) = 0 Then Exit For
  Next pos1
  If pos1 <> 1 Then pos1 = pos1 + 1 ' Von Zeichen vor Wortbeginn wieder auf das
                                    ' nchste Zeichen (Wortbeginn) aufsteigen
  
  ' 2) Nach gleicher Manier das Ende des Wortes suchen
  intLaenge = Len(strText)
  For pos2 = pos To intLaenge
    strChar = Mid$(strText, pos2, 1)
    ' Ist dieser Buchstabe Teil eines Wortes?
    If InStr(mvarCharsAllowed, strChar) = 0 Then Exit For
  Next pos2
  If pos2 <> Len(strText) Then pos2 = pos2 - 1 ' Von Zeichen hinter dem Wort wieder eine Stelle rckwrts
  
  ' Aus Anfangs- und Endposition ergibt sich nach Plausibilittsprfung das Wort
  If pos1 <= pos2 Then WordFromPoint = Mid$(strText, pos1, pos2 - pos1 + 1)
      
End Function


Private Sub Class_Initialize()
Dim i As Integer

  ' Default-Definition der Zeichen, die als Teil eines Wortes gelten drfen:
  ' Kleinbuchstaben:
  For i = Asc("a") To Asc("z")
    mvarCharsAllowed = mvarCharsAllowed & Chr$(i)
  Next i
  ' Grobuchstaben:
  For i = Asc("A") To Asc("Z")
    mvarCharsAllowed = mvarCharsAllowed & Chr$(i)
  Next i
  ' Ziffern:
  For i = Asc("0") To Asc("9")
    mvarCharsAllowed = mvarCharsAllowed & Chr$(i)
  Next i
  ' Umlaute und Sonderzeichen:
  mvarCharsAllowed = mvarCharsAllowed & "_@"

End Sub


Public Property Let CharsAllowed(ByVal vData As String)
    mvarCharsAllowed = vData
End Property


Public Property Get CharsAllowed() As String
    CharsAllowed = mvarCharsAllowed
End Property

